@@ -1,7 +1,7 @@
 package App::shcompgen;
 
-our $DATE = '2016-10-21'; # DATE
-our $VERSION = '0.20'; # VERSION
+our $DATE = '2016-10-25'; # DATE
+our $VERSION = '0.21'; # VERSION
 
 use 5.010001;
 use strict;
@@ -41,7 +41,7 @@ my $_complete_prog = sub {
     }
 };
 
-our @supported_shells = qw(bash fish); # XXX tcsh zsh
+our @supported_shells = qw(bash fish zsh); # XXX tcsh
 our %shell_arg = (
     shell => {
         summary => 'Override guessing and select shell manually',
@@ -104,16 +104,15 @@ _
     #    schema  => ['array*', of => 'str*'],
     #},
 
-    #zsh_global_dir => {
-    #    summary => 'Directory to put completions scripts',
-    #    schema  => ['array*', of => 'str*'],
-    #    schema  => 'str*',
-    #    default => [],
-    #},
-    #zsh_per_user_dir => {
-    #    summary => 'Directory to put completions scripts',
-    #    schema  => ['array*', of => 'str*'],
-    #},
+    zsh_global_dir => {
+        summary => 'Directory to put completions scripts',
+        schema  => ['array*', of => 'str*'],
+        default => ['/usr/local/share/zsh/site-functions'],
+    },
+    zsh_per_user_dir => {
+        summary => 'Directory to put completions scripts',
+        schema  => ['array*', of => 'str*'],
+    },
 );
 
 sub _all_exec_in_PATH {
@@ -138,6 +137,7 @@ sub _set_args_defaults {
         my $sh = Shell::Guess->running_shell;
         my $n = $sh->{name};
         $n = "zsh" if $n eq 'z';
+        $n = "bash" if $n eq 'bourne'; # under make
         $args->{shell} = $n;
     }
     unless ($args->{shell} ~~ @supported_shells) {
@@ -154,8 +154,8 @@ sub _set_args_defaults {
     $args->{fish_per_user_dir} //= ["$ENV{HOME}/.config/fish/completions"];
     #$args->{tcsh_global_dir}   //= [];
     #$args->{tcsh_per_user_dir} //= ["$ENV{HOME}/.config/tcsh/completions"];
-    #$args->{zsh_global_dir}    //= [];
-    #$args->{zsh_per_user_dir}  //= ["$ENV{HOME}/.config/zsh/completions"];
+    $args->{zsh_global_dir}    //= ['/usr/local/share/zsh/site-functions'];
+    $args->{zsh_per_user_dir}  //= ["$ENV{HOME}/.config/zsh/completions"];
     [200];
 }
 
@@ -664,9 +664,11 @@ sub init {
     my $init_location;
     my $init_script;
     my $init_script_path;
+
+    $dirs = _completion_scripts_dirs(%args);
+
     if ($shell eq 'bash') {
         $init_location = $global ? "/etc/bash.bashrc" : "~/.bashrc";
-        $dirs = _completion_scripts_dirs(%args);
         $init_script = <<_;
 # generated by shcompgen version $App::shcompgen::VERSION
 _
@@ -708,9 +710,33 @@ _
         } else {
             $init_script_path = "$ENV{HOME}/.config/shcompgen.bashrc";
         }
+        $instruction .= "Please put this into your $init_location:".
+            "\n\n" . " . $init_script_path\n\n";
+    } elsif ($shell eq 'zsh') {
+        $init_location = $global ? "/etc/zsh/zshrc" : "~/.zshrc";
+        $init_script = <<_;
+# generated by shcompgen version $App::shcompgen::VERSION
+_
+        $init_script .= <<'_';
+local added_dir
+for d in ~/.config/zsh/completions; do
+  if [[ ${fpath[(i)$d]} == "" || ${fpath[(i)$d]} -gt ${#fpath} ]]; then
+    fpath=($d $fpath)
+    added_dir=1
+  fi
+done
+if [[ $added_dir == 1 ]]; then compinit; fi
+_
+
+        if ($global) {
+            $init_script_path = "/etc/shcompgen.zshrc";
+        } else {
+            $init_script_path = "$ENV{HOME}/.config/shcompgen.zshrc";
+        }
+        $instruction .= "Please put this into your $init_location:".
+            "\n\n" . " . $init_script_path\n\n";
     } elsif ($shell eq 'fish') {
-        # nothing to do, should be ready
-        return [200, "OK"];
+        # nothing to do, ready by default
     } else {
         return [412, "Shell '$shell' not yet supported"];
     }
@@ -718,15 +744,16 @@ _
     for my $dir (@$dirs) {
         unless (-d $dir) {
             require File::Path;
+            $log->tracef("Creating directory %s ...", $dir);
             File::Path::make_path($dir)
                   or return [500, "Can't create $dir: $!"];
             $instruction .= "Directory '$dir' created.\n\n";
         }
     }
 
-    write_text($init_script_path, $init_script);
-    $instruction .= "Please put this into your $init_location:".
-        "\n\n" . " . $init_script_path\n\n";
+    if ($init_script) {
+        write_text($init_script_path, $init_script);
+    }
 
     $instruction = "Congratulations, shcompgen initialization is successful.".
         "\n\n$instruction";
@@ -899,7 +926,7 @@ App::shcompgen - Generate shell completion scripts
 
 =head1 VERSION
 
-This document describes version 0.20 of App::shcompgen (from Perl distribution App-shcompgen), released on 2016-10-21.
+This document describes version 0.21 of App::shcompgen (from Perl distribution App-shcompgen), released on 2016-10-25.
 
 =head1 FUNCTIONS
 
@@ -1001,6 +1028,14 @@ Override guessing and select shell manually.
 
 Output completion script to STDOUT.
 
+=item * B<zsh_global_dir> => I<array[str]> (default: ["/usr/local/share/zsh/site-functions"])
+
+Directory to put completions scripts.
+
+=item * B<zsh_per_user_dir> => I<array[str]>
+
+Directory to put completions scripts.
+
 =back
 
 Returns an enveloped result (an array).
@@ -1080,6 +1115,14 @@ that and manually select which.
 
 Override guessing and select shell manually.
 
+=item * B<zsh_global_dir> => I<array[str]> (default: ["/usr/local/share/zsh/site-functions"])
+
+Directory to put completions scripts.
+
+=item * B<zsh_per_user_dir> => I<array[str]>
+
+Directory to put completions scripts.
+
 =back
 
 Returns an enveloped result (an array).
@@ -1138,6 +1181,14 @@ that and manually select which.
 
 Override guessing and select shell manually.
 
+=item * B<zsh_global_dir> => I<array[str]> (default: ["/usr/local/share/zsh/site-functions"])
+
+Directory to put completions scripts.
+
+=item * B<zsh_per_user_dir> => I<array[str]>
+
+Directory to put completions scripts.
+
 =back
 
 Returns an enveloped result (an array).
@@ -1201,6 +1252,14 @@ searched from PATH.
 
 Override guessing and select shell manually.
 
+=item * B<zsh_global_dir> => I<array[str]> (default: ["/usr/local/share/zsh/site-functions"])
+
+Directory to put completions scripts.
+
+=item * B<zsh_per_user_dir> => I<array[str]>
+
+Directory to put completions scripts.
+
 =back
 
 Returns an enveloped result (an array).
